/*
 *    regular expression module.
 *
 * Copyright (c) 1999, 2000 SASAKI Shunsuke.
 * All rights reserved. 
 *
 * Redistribution and use in source and binary forms, with or without
 * modification, are permitted provided that the following conditions
 * are met:
 * 1. Redistributions of source code must retain the above copyright
 *    notice, this list of conditions, and the following disclaimer,
 *    without modification, immediately at the beginning of the file.
 * 2. The name of the author may not be used to endorse or promote products
 *    derived from this software without specific prior written permission.
 *
 * Where this Software is combined with software released under the terms of 
 * the GNU Public License ("GPL") and the terms of the GPL would require the 
 * combined work to also be released under the terms of the GPL, the terms
 * and conditions of this License will apply in addition to those of the
 * GPL with the exception of any terms or conditions of this License that
 * conflict with, or are expressly prohibited by, the GPL.
 *
 * THIS SOFTWARE IS PROVIDED BY THE AUTHOR AND CONTRIBUTORS ``AS IS'' AND
 * ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE
 * IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE
 * ARE DISCLAIMED. IN NO EVENT SHALL THE AUTHOR OR CONTRIBUTORS BE LIABLE FOR
 * ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL
 * DAMAGES (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS
 * OR SERVICES; LOSS OF USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION)
 * HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT
 * LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY
 * OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF
 * SUCH DAMAGE.
 */

#include	"config.h"

#include	<stdio.h>
#include	<string.h>
#include	<sys/types.h>

#include	"generic.h"
#include	"regexp.h"


/*  */

int 	regexp_exec(const char *s,const char *t,regm_t rms[],size_t n)
{
	int 	i;
	const char	*p;
	const char	*ss;

//int 	j;

	ss=s;
	for (i=0;;++i)
		{
		 p=strstr(s,t);
		 if (p==NULL)
		 	break;
		 s=p+1;
		 if (i<n)
		 	{
		 	 rms[i].rm_eo=rms[i].rm_so=p-ss;
		 	 rms[i].rm_eo+=strlen(t);
		 	}
		}

//for (j=0;j<i;++j)
//	{if (j<n)fprintf(stderr,"j(%d:%d) ",j,rms[j].rm_so,rms[j].rm_eo);}
//fprintf(stderr,"\n");

	return i;
}


bool	regexp_strck(const char *s,const char *t)
{
	return regexp_exec(s,t,NULL,0)>0;
}

bool	regexp_seeknext(const char *s,const char *t,int x,regm_t *rmp)
{
	regm_t	*rms;
	int 	i,n,len;
	bool	f;

	len=strlen(s);
	if (x>=len)
		return FALSE;

	rms=(regm_t *)malloc(sizeof(regm_t)*len);
	if (rms==NULL)
		{
//		 inkey_wait(MEMORY_ERR_MSG);
		 return FALSE;
		}

	f=FALSE;
	n=min(regexp_exec(s,t,rms,len),len);
	for (i=0;i<n;++i)
		{
		 if (rms[i].rm_so>=x)
		 	{
		 	 rmp->rm_so=rms[i].rm_so;
		 	 rmp->rm_eo=rms[i].rm_eo;
		 	 f=TRUE;
		 	 break;
		 	}
		}
	free(rms);
	return f;

}

bool	regexp_seekprev(const char *s,const char *t,int x,regm_t *rmp)
{
	regm_t	*rms;
	int 	i,n;
	bool	f;

	if (x<=0)
		return FALSE;
	if (x>strlen(s))
		x=strlen(s);

	++x;

	rms=(regm_t *)malloc(sizeof(regm_t)*x);
	if (rms==NULL)
		{
//		 inkey_wait(MEMORY_ERR_MSG);
		 return FALSE;
		}

	f=FALSE;
	n=min(regexp_exec(s,t,rms,x),x);

//fprintf(stderr,"!%d %d\n",n,x);
	for (i=n;i>0;)
		{
		 --i;
		 if (rms[i].rm_so<x)
		 	{
		 	 rmp->rm_so=rms[i].rm_so;
		 	 rmp->rm_eo=rms[i].rm_eo;
//fprintf(stderr,"*%d %d:%d\n",i,rmp->rm_so,rmp->rm_eo);
		 	 f=TRUE;
		 	 break;
		 	}
		}
	free(rms);
	return f;
}

